{%- for css_file in css_files %}
<link rel="stylesheet" href="{{css_file}}" type="text/css" />
{%- endfor %}
{%- for js_file in js_files %}
<script type="text/javascript" src="{{ js_file }}"></script>
{%- endfor %}
{%- if bjs_script %}
<script>{{ bjs_script }}</script>
{%- endif %}
