  </div>
   {% if enable_codepen %}
   <form action="https://codepen.io/pen/define" method="POST" target="_blank">
     <input value="Try '{{ title }}' on CodePen" type="submit" class="codepen-open"/>
   </form>
   {% endif %}

   <p>The code above generates the following plot:</p>
   <script>
     (function() {
         {{ js_source }}
     })();
   </script>
</div>
