export const left: string
export const right: string
export const above: string
export const below: string
export const open: string
export const handle: string
export const contents: string
export const resizing: string
export const toggle: string
export const chevron: string
export const resizable: string
const __css__: string
export default __css__
