export const hidden: string
export const title: string
export const label: string
export const grid: string
export const vertical: string
export const item: string
export const styled: string
export const interactive: string
export const glyph: string
export const overlay: string
export const inactive: string
const __css__: string
export default __css__
