#-----------------------------------------------------------------------------
# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
# All rights reserved.
#
# The full license is in the file LICENSE.txt, distributed with this software.
#-----------------------------------------------------------------------------
''' Provide a collection of general utilities useful for implementing Bokeh
functionality.

.. _bokeh.util.browser:

``bokeh.util.browser``
----------------------

.. automodule:: bokeh.util.browser
  :members:

.. _bokeh.util.callback_manager:

``bokeh.util.callback_manager``
-------------------------------

.. automodule:: bokeh.util.callback_manager
  :members:

.. _bokeh.util.compiler:

``bokeh.util.compiler``
-----------------------

.. automodule:: bokeh.util.compiler
  :members:

.. _bokeh.util.dependencies:

``bokeh.util.dependencies``
---------------------------

.. automodule:: bokeh.util.dependencies
  :members:

.. _bokeh.util.deprecation:

``bokeh.util.deprecation``
--------------------------

.. automodule:: bokeh.util.deprecation

.. _bokeh.util.functions:

``bokeh.util.functions``
------------------------

.. automodule:: bokeh.util.functions
  :members:

``bokeh.util.hex``
------------------

.. automodule:: bokeh.util.hex
  :members:

.. _bokeh.util.info:

``bokeh.util.info``
----------------------

.. automodule:: bokeh.util.info
  :members:

.. _bokeh.util.logconfig:

``bokeh.util.logconfig``
------------------------

.. automodule:: bokeh.util.logconfig
  :members:

.. _bokeh.util.options:

``bokeh.util.options``
----------------------

.. automodule:: bokeh.util.options
  :members:

.. _bokeh.util.paths:

``bokeh.util.paths``
--------------------

.. automodule:: bokeh.util.paths
  :members:

.. _bokeh.util.serialization:

``bokeh.util.serialization``
----------------------------

.. automodule:: bokeh.util.serialization
  :members:

.. _bokeh.util.token:

``bokeh.util.token``
--------------------

.. automodule:: bokeh.util.token
  :members:

.. _bokeh.util.strings:

``bokeh.util.strings``
----------------------

.. automodule:: bokeh.util.strings
  :members:

.. _bokeh.util.tornado:

``bokeh.util.tornado``
----------------------

.. automodule:: bokeh.util.tornado
  :members:

  .. _bokeh.util.terminal:

``bokeh.util.terminal``
-----------------------

.. automodule:: bokeh.util.terminal
  :members:

.. _bokeh.util.version:

``bokeh.util.version``
----------------------

.. automodule:: bokeh.util.version
  :members:

.. _bokeh.util.warnings:

``bokeh.util.warnings``
-----------------------

.. automodule:: bokeh.util.warnings
  :members:

'''
