export const nullish: string
export const token: string
export const boolean: string
export const number: string
export const string: string
export const symbol: string
export const model: string
export const attr: string
export const array: string
export const object: string
export const iterable: string
const __css__: string
export default __css__
