export const bar: string
export const value: string
export const disabled: string
export const horizontal: string
export const vertical: string
export const reversed: string
export const label: string
export const hidden: string
export const indeterminate: string
const __css__: string
export default __css__
