export const item: string
export const divider: string
export const check: string
export const icon: string
export const label: string
export const shortcut: string
export const chevron: string
export const disabled: string
export const menu: string
export const checkable: string
export const checked: string
const __css__: string
export default __css__
