export const icon: string
export const body: string
export const disabled: string
export const bar: string
export const knob: string
export const active: string
const __css__: string
export default __css__
